#PKfit main menu
PKmenu <- function() 
{
  cat("\n")
  file.menu <- c("Normal Fitting", 
                 "Simulation", 
                 "Quit")
  pick <- menu(file.menu, title = "<< PK startup menu >>")  
  if (pick == 1){
     cat("\n\n")
     nor.fit()
  } 
  else if (pick == 2){
     cat("\n\n") 
     PK.sim()
  } 
  else if (pick == 3){
     cat("\nQuit !!\n\n")     
     
  }  
}

#Normal fitting
nor.fit <- function(PKindex)
{
  file.menu <- c("Data Manipulation",
                 "Selection of PK Model",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Normal Fitting >>")
  if (pick == 1){
     cat("\n\n")      
     data.manipulate()
  } 
  else if (pick == 2){
     cat("\n\n") 
     PK.fit(PKindex)
  }      
  else if (pick == 3){
     cat("\n\n") 
     PKmenu()
  }  
}

#Data manipulation
data.manipulate <- function()
{
  file.menu <- c("Load Data Files (.CSV)", 
                 "Load Data Files (.RData)", 
                 "Key In Data", 
                 "Go Back One Upper Level")  
  pick <- menu(file.menu, title = "<< Data edit >>")
  
  if (pick == 1){
     cat("\n\n<< Enter data file name(.csv) >>\n")
     cat("<< Data should be consist of subject number, time, and concnetration >>\n")
     cat("<< The file should be put in the working directory >>\n")
     PK.file <-readline()
     dataExt<- ".csv"
     PK.file<-paste(PK.file,dataExt,sep="")
     cnames<-c("Subject", "time", "conc")
     PKindex<-read.csv(PK.file,header=TRUE,sep=",",row.names=NULL,col.names=cnames)
     PKindex<-edit(PKindex)
     show(PKindex)
     cat("\n\n")
     return(nor.fit(PKindex))   
  } 
  else if (pick == 2){
     cat("\nEnter data file name\n\n") 
     PKname <-readline()
     dataExt<- ".RData"
     PKname<-paste(PKname,dataExt,sep="")
     load(PKname)
     PKindex<-edit(PKindex)
     show(PKindex)
     save(PKindex,file=PKname)
     cat("\n")
     nor.fit(PKindex)
  }   
  else if (pick == 3){
     cat("\n\n") 
     PKindex<-data.frame(Subject=c(1),time=c(0),conc=c(0))
     PKindex<-edit(PKindex)
     show(PKindex)     
     cat("\n\n")
     cat("\nSave data (y/n) ?\n\n")
     ans<-readline()
     cat("\n")
     if (ans == "n"){
        return(nor.fit(PKindex))
        }
     else {
        cat("Enter name you want to call this data\n\n")
        PKname <-readline() 
        dataExt<- ".RData"
        PKname<-paste(PKname,dataExt,sep="")
        save(PKindex,file=PKname)
        cat("\n")  
        return(nor.fit(PKindex))
       }
  } 
  else if (pick == 4){
     cat("\n\n") 
     return(nor.fit())
  } 
}

#PK model option
PK.fit <- function(PKindex)
{
  file.menu <- c("One-Compartment PK Model", 
                 "Two-Compartment PK Model",
                 "Macroconstant Exponential Functions",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Selection of PK Model >>")
  if (pick== 1){
     cat("\n\n")  
     one.list(PKindex)
  }     
  else if (pick == 2){
     cat("\n\n")
     two.list(PKindex)
  }
  else if (pick == 3){
     cat("\n\n")
     macro(PKindex)
  }
  else if (pick == 4){
     cat("\n\n")
     nor.fit()
  }        
}

#------------Normal fitting menu----------------
one.list <- function(PKindex)
{
  file.menu <- c("IV (Bolus or Infusion) Route", 
                 "Non IV Route",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< One-Compartment Model >>")
  if (pick == 1){
     cat("\n\n")  
     iv.route(PKindex)
  }
  else if (pick == 2){
     cat("\n\n")
     noniv.route(PKindex)
  }
  else if (pick == 3){
     cat("\n\n")
     PK.fit()
  }              
}

two.list <- function(PKindex)
{
  file.menu <- c("IV-Bolus, and Single-Dose", 
                 "IV-Infusion, and Single-Dose",
                 "Extravascular, Single-Dose, and First-Ordered Absorption without Lag Time",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Two-Compartment Model >>")
  if (pick == 1){
     cat("\n\n")  
     fbolus2(PKindex)
  }
  else if (pick == 2){
     cat("\n\n")
     finfu2(PKindex)
  }
  else if (pick == 3){
     cat("\n\n")
     ffirst2(PKindex)
  }
  else if (pick == 4){
     cat("\n\n")
     PK.fit()
  }
}

PK.sim <- function()
{
  file.menu <- c("One-Compartment Model: IV (Bolus or Infusion) Route", 
                 "One-Compartment Model: Non IV Route",
                 "Two-Compartment Model",
                 "Macroconstant Exponential Functions",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Selection of PK Model >>")
  if (pick == 1){
     cat("\n\n")  
     sone.iv.route()
  }
  else if (pick == 2){
     cat("\n\n")
     sone.noniv.route()
  }
  else if (pick == 3){
     cat("\n\n")
     stwo.all()
  }      
  else if (pick == 4){
     cat("\n\n")
     smacro()
  }
  else if (pick == 5){
     cat("\n\n")
     PKmenu()
  }
}

iv.route <- function(PKindex)
{
  file.menu <- c("IV-Bolus, and Single-Dose", 
                 "IV-Bolus, Single-Dose, and Nonlinear Elimination", 
                 "IV-Infusion, and Single-Dose",
                 "IV-Infusion, Single-Dose, and Nonlinear Elimination",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< IV (Bolus or Infusion) Route >>")
  if (pick ==1){
     cat("\n\n")
     fbolus1(PKindex)
  }
  else if (pick == 2){
     cat("\n\n") 
     fbolus.mm(PKindex)
  }
  else if (pick == 3){
     cat("\n\n") 
     finfu1(PKindex)
  }
  else if (pick == 4){
     cat("\n\n") 
     finfu.mm(PKindex)
  }
  else if (pick == 5){
     cat("\n\n") 
     one.list(PKindex)
  }
}

noniv.route <- function(PKindex)
{
  file.menu <- c("Single-Dose, and First-Ordered Absorption with Lag Time",
                 "Single-Dose, and First-Ordered Absorption without Lag Time",
                 "Single-Dose, and Zero-Ordered Absorption without Lag Time",
                 "Single-Dose, First-Ordered Absorption, and Nonlinear Elimination with Lag Time ",
                 "Single-Dose, First-Ordered Absorption, and Nonlinear Elimination without Lag Time",
                 "Single-Dose, Zero-Ordered Absorption, and Nonlinear Elimination without Lag Time",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Non IV Route >>")
  if (pick == 1){
     cat("\n\n") 
     ffirst.lag(PKindex)
  }
  else if (pick == 2){
     cat("\n\n") 
     ffirst.nolag(PKindex)
  }
  else if (pick == 3){
     cat("\n\n") 
     fzero.nolag(PKindex)
  }
  else if (pick == 4){
     cat("\n\n") 
     ffirst.lagm(PKindex)
  }
  else if (pick == 5){
     cat("\n\n") 
     ffirst.nolagm(PKindex)
  }
  else if (pick == 6){
     cat("\n\n") 
     fzero.nolagm(PKindex)
  }
  else if (pick == 7){
     cat("\n\n") 
     one.list(PKindex)
  }
}

macro <- function(PKindex)
{
  file.menu <- c("One-Exponential Term", 
                 "Two-Exponential Term",
                 "Three-Exponential Term",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Macroconstant Exponential Functions >>")
  if (pick ==1){
     cat("\n\n")
     fmacro.one(PKindex)
  }
  else if (pick == 2){
     cat("\n\n") 
     fmacro.two(PKindex)
  }
  else if (pick == 3){
     cat("\n\n") 
     fmacro.three(PKindex)
  }
  else if (pick == 4){
     cat("\n\n") 
     PK.fit()
  }
}

#------------Simulation menu----------------
sone.iv.route <- function()
{
  file.menu <- c("IV-Bolus, and Single-Dose", 
                 "IV-Bolus, Single-Dose, and Nonlinear Elimination", 
                 "IV-Infusion, and Single-Dose",
                 "IV-Infusion, Single-Dose, and Nonlinear Elimination",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< IV (Bolus or Infusion) Route >>")
  if (pick ==1 ){
     cat("\n\n")
     sbolus1()
  }
  else if (pick == 2){
     cat("\n\n") 
     sbolus.mm()
  }
  else if (pick == 3){
     cat("\n\n") 
     sinfu1()
  }
  else if (pick == 4){
     cat("\n\n") 
     sinfu.mm()
  }
  else if (pick == 5){
     cat("\n\n") 
     PK.sim()
  }
}

sone.noniv.route <- function()
{
  file.menu <- c("Single-Dose, and First-Ordered Absorption with Lag Time",
                 "Single-Dose, and First-Ordered Absorption without Lag Time",
                 "Single-Dose, and Zero-Ordered Absorption without Lag Time",
                 "Single-Dose, First-Ordered Absorption, and Nonlinear Elimination with Lag Time ",
                 "Single-Dose, First-Ordered Absorption, and Nonlinear Elimination without Lag Time",
                 "Single-Dose, Zero-Ordered Absorption, and Nonlinear Elimination without Lag Time",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Non IV Route >>")
  if (pick == 1){
     cat("\n\n") 
     sfirst.lag()
  }
  else if (pick == 2){
     cat("\n\n") 
     sfirst.nolag()
  }
  else if (pick == 3){
     cat("\n\n") 
     szero.nolag()
  }
  else if (pick == 4){
     cat("\n\n") 
     sfirst.lagm()
  }
  else if (pick == 5){
     cat("\n\n") 
     sfirst.nolagm()
  }
  else if (pick == 6){
     cat("\n\n") 
     szero.nolagm()
  }
  else if (pick == 7){
     cat("\n\n") 
     PK.sim()
  }
}

smacro <- function()
{
  file.menu <- c("One-Exponential Term", 
                 "Two-Exponential Term", 
                 "Three-Exponential Term",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Macroconstant Exponential Functions >>")
  if (pick ==1){
     cat("\n\n")
     smacro.one()
  }
  else if (pick == 2){
     cat("\n\n") 
     smacro.two()
  }
  else if (pick == 3){
     cat("\n\n") 
     smacro.three()
  }
  else if (pick == 4){
     cat("\n\n") 
     PK.sim()
  }
}

stwo.all <- function()
{
  file.menu <- c("IV-Bolus, and Single-Dose", 
                 "IV-Infusion, and Single-Dose",
                 "Extravascular, Single-Dose, and First-Ordered Absorption without Lag Time",
                 "Go Back One Upper Level")
  pick <- menu(file.menu, title = "<< Two-Compartment Model >>")
  if (pick ==1){
     cat("\n\n")
     sbolus2()
  }
  else if (pick == 2){
     cat("\n\n") 
     sinfu2()
  }
  else if (pick == 3){
     cat("\n\n") 
     sfirst2()
  }
  else if (pick == 4){
     cat("\n\n") 
     PK.sim()
  }  
}

#---------------plot for linear----------------
plotting.lin <- function (PKindex, fm, i, pick, coef, xaxis, yaxis)
{               
  j<-1:length(PKindex$time[PKindex$Subject==i])
  x<-PKindex$time[PKindex$Subject==i]
  y<-PKindex$conc[PKindex$Subject==i]
        
 #Calculated concentration
  cal<-predict(fm,list(time=x))
    
 #Weighted residuals
  if (pick ==1){
     wei<-ifelse(y[j]==0.0, 0, y[j]-cal[j])
  }
  else if (pick ==2){
     wei<-ifelse(y[j]==0.0, 0, sqrt(1/(y[j]))*(y[j]-cal[j]))
  }
  else if (pick ==3){  
     wei<-ifelse(y[j]==0.0, 0, sqrt(1/((y[j])^2))*(y[j]-cal[j]))
  }
  
 #calculate AUC and AUMC   
  add<-function(time,conc){
     auc<-0 ; aumc<-0
     for(i in 2:length(time)) {
     auc[i]<-1/2*(time[i]-time[i-1])*(conc[i]+conc[i-1])
     auc[i]<-auc[i]+auc[i-1]
     aumc[i]<-1/2*(time[i]-time[i-1])*(conc[i]*time[i]+conc[i-1]*time[i-1])
     aumc[i]<-aumc[i]+aumc[i-1]
  }
     return(list(auc=auc,aumc=aumc))
  }
  add<-add(x,y)
  AUC<-add$auc
  AUMC<-add$aumc
        
 #Output   
  cat("<< Output >>\n")  
  output<-data.frame(x,y,cal,wei,AUC,AUMC)
  colnames(output)<-list("time","Observed","Calculated","Wtd Residuals","AUC","AUMC")
  show(output)  
  
 #AUC (0 to infinity)              
  cat("\n<< AUC (0 to infinity) computed by trapezoidal rule >>\n\n")
  auc.infinity<-y[length(y)]/coef[1,1]
  auc<-AUC[length(y)]+auc.infinity
  show(auc) 
  
 #AUMC (0 to infinity) 
  cat("\n<< AUMC (0 to infinity) computed by trapezoidal rule >>\n\n")
  aumc.infinity<-(x[length(x)]*y[length(y)])/coef[1,1]+x[length(x)]/((coef[1,1])^2)
  aumc<-AUMC[length(y)]+aumc.infinity
  show(aumc)   
  cat("\n")
         
  aicllsbc(fm)
  cat("\n\n")
    
 #Divide plot console into four parts
  windows()
  par(mfrow=c(2,2))
  main<-paste(c("Subject", i ,"plot"),collapse=" ")
     
 #Linear plot
  plot(y~x,data=PKindex,type='p',main=main, 
  xlab=xaxis, ylab=yaxis,pch=15,col="black",bty="l",
  font.lab=2,cex.lab=1,cex.axis=1,cex.main=1) 
  lines(x,predict(fm,list(time=x)),type="l",lty=1,
  col="firebrick3",lwd="2")
  mtext("Linear",side=3,cex=0.88)
    
 #Semi-log plot
  plot(x,y,log="y",type='p',main=main,
  xlab=xaxis, ylab=yaxis,pch=15,col="black",bty="l",
  font.lab=2,cex.lab=1,cex.axis=1,cex.main=1) 
  lines(x,predict(fm,list(time=x)),type="l",lty=1,
  col="firebrick3",lwd="2")
  mtext("Semi-log",side=3,cex=0.88)
     
 #Residual plot, time vs weighted residual
  plot(x,wei,pch=15,col="blue",bty="l",xlab=xaxis,
  ylab="Weighted Residual",main="Residual Plots",cex.lab=1,
  cex.axis=1,cex.main=1,font.lab=2)
  abline(h=0,lwd=2,col="black",lty=2)
    
 #Residual plot, calculated concentration vs weigthed residual
  plot(cal,wei,pch=15,col="blue",bty="l",xlab="Calc Cp(i)",
  ylab="Weighted Residual",main="Residual Plots",cex.lab=1,
  cex.axis=1,cex.main=1,font.lab=2)
  abline(h=0,lwd=2,col="black",lty=2)          
}    

#---------------plot for nonlinear----------------
plotting.non <- function (PKindex, fm, i, pick, xaxis, yaxis)
{  
  options(warn=-1)
              
  j<-1:length(PKindex$time[PKindex$Subject==i])
  x<-PKindex$time[PKindex$Subject==i]
  y<-PKindex$conc[PKindex$Subject==i]
   
 #Calculated concentration
  cal<-predict(fm,list(time=x))
    
 #Weighted residuals
  if (pick ==1){
     wei<-ifelse(y[j]==0.0, 0, y[j]-cal[j])
  }
  else if (pick ==2){
     wei<-ifelse(y[j]==0.0, 0, sqrt(1/(y[j]))*(y[j]-cal[j]))
  }
  else if (pick ==3){  
     wei<-ifelse(y[j]==0.0, 0, sqrt(1/((y[j])^2))*(y[j]-cal[j]))
  }
                   
 #calculate AUC and AUMC       
  add<-function(time,conc){
     auc<-0 ; aumc<-0
     for(i in 2:length(time)) {
     auc[i]<-1/2*(time[i]-time[i-1])*(conc[i]+conc[i-1])
     auc[i]<-auc[i]+auc[i-1]
     aumc[i]<-1/2*(time[i]-time[i-1])*(conc[i]*time[i]+conc[i-1]*time[i-1])
     aumc[i]<-aumc[i]+aumc[i-1]
  }
     return(list(auc=auc,aumc=aumc))
  }
  add<-add(x,y)
  AUC<-add$auc
  AUMC<-add$aumc
              
 #Output  
  cat("<< Output >>\n\n")      
  output<-data.frame(x,y,cal,wei,AUC,AUMC)
  colnames(output)<-list("time","Observed","Calculated","Wtd Residuals","AUC","AUMC")
  show(output)  
  cat("\n") 
        
  aicllsbc(fm)
  cat("\n\n")
    
 #Divide plot console into four parts
  windows()
  par(mfrow=c(2,2))
  main<-paste(c("Subject", i ,"plot"),collapse=" ")
     
 #Linear plot
  plot(y~x,data=PKindex,type='p',main=main, 
  xlab="Time", ylab="Concentration",pch=15,col="black",bty="l",
  font.lab=2,cex.lab=1,cex.axis=1,cex.main=1) 
  lines(x,predict(fm,list(time=x)),type="l",lty=1,
  col="firebrick3",lwd="2")
  mtext("Linear",side=3,cex=0.88)
    
 #Semi-log plot
  plot(x,y,log="y",type='p',main=main,
  xlab="Time", ylab="Concentration",pch=15,col="black",bty="l",
  font.lab=2,cex.lab=1,cex.axis=1,cex.main=1) 
  lines(x,predict(fm,list(time=x)),type="l",lty=1,
  col="firebrick3",lwd="2")
  mtext("Semi-log",side=3,cex=0.88)
     
 #Residual plot, time vs weighted residual----- 
  plot(x,wei,pch=15,col="blue",bty="l",xlab="Time",
  ylab="Weighted Residual",main="Residual Plots",cex.lab=1,
  cex.axis=1,cex.main=1,font.lab=2)
  abline(h=0,lwd=2,col="black",lty=2)
    
 #Residual plot, calcukated concentration vs weigthed residual-----
  plot(cal,wei,pch=15,col="blue",bty="l",xlab="Calc Cp(i)",
  ylab="Weighted Residual",main="Residual Plots",cex.lab=1,
  cex.axis=1,cex.main=1,font.lab=2)
  abline(h=0,lwd=2,col="black",lty=2)
}    

#----------------plot for simulation----------------
plotting.sim <- function(i,x,y)
{
  main<-paste(c("Subject", i ,"plot"),collapse=" ")
  options(warn=-1)
  windows()
  par(mfrow=c(2,2))
   
 #linear plot
  plot(y~x,type='p',main=main, 
  xlab="Time",ylab="Concentration",pch=15,col="black",bty="l",
  font.lab=2,cex.lab=1,cex.axis=1,cex.main=1) 
   text("Linear",side=3,cex=0.88)

 #semi-log plot
  plot(x,y,log="y",type='p',main=main,
  xlab="Time",ylab="Concentration",pch=15,col="black",bty="l",
  font.lab=2,cex.lab=1,cex.axis=1,cex.main=1) 
  mtext("Semi-log",side=3,cex=0.88) 
  
  cat("\n\n")
}

#Estimate model fitting: AIC, Log likelihood, SBC
aicllsbc <- function(fm)
{   
  cat("\n") 
  cat("<< Akaike's Information Criterion (AIC) >>\n\n")
  show(AIC(fm))
    
  cat("\n<< Log likelihood >>\n\n")
  show(logLik(fm))
    
  cat("\n<< Schwarz's Bayesian Criterion (SBC) >>\n\n")
  show(BIC(fm))
  cat("\n")     
    
 #Summary the results of nls
  print(summary(fm))      
}  


savefile<-function(PKindex)  
{
  cat("\n\n")
  cat("\nSave data as a .RData file (y/n) ?\n\n")
  ans<-readline()
  cat("\n")
     if (ans == "n"){
     }
  else {
     cat("Enter name you want to call this data\n\n")
     PKname <-readline() 
     dataExt<- ".RData"
     PKname<-paste(PKname,dataExt,sep="")
     save(PKindex,file=PKname)
     cat("\n")  
    }
}


#for iv bolus 
sbolus1.out<-function(PKtime,kel,Vd,defun,par1,par2,Dose,i)
{
  time<-PKtime$time
  parms<-c(kel=kel,Vd=Vd) 
  C1.lsoda<-data.frame(lsoda(Dose/Vd,c(0,time),defun,parms)) 
  cat("\n")
  sim1<-kel
  sim2<-Vd
  sim<-matrix(c(sim1,sim2,par1,par2),2,2)
  dimnames(sim)<-list(c("kel","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),2])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for iv bolus nonlinear elimination 
sbolus.mm.out<-function(PKtime,Vm,Km,Vd,defun,par1,par2,par3,Dose,i)
{ 
  time<-PKtime$time
  parms<-c(Vm=Vm,Km=Km,Vd=Vd)  
  C1.lsoda<-data.frame(lsoda(Dose/Vd,c(0,time),defun,parms))     
  cat("\n")
  sim1<-Vm
  sim2<-Km
  sim3<-Vd
  sim<-matrix(c(sim1,sim2,sim3,par1,par2,par3),3,2)
  dimnames(sim)<-list(c("Vm","Km","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),2])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for iv infusion 
sinfu1.out<-function(PKtime,kel,Vd,defun,par1,par2,Dose,i)
{
  time<-PKtime$time
  parms<-c(kel=kel,Vd=Vd) 
  C1.lsoda<-data.frame(lsoda(0,c(0,time),defun,parms)) 
  cat("\n")
  sim1<-kel
  sim2<-Vd
  sim<-matrix(c(sim1,sim2,par1,par2),2,2)
  dimnames(sim)<-list(c("kel","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),2])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for iv infusion nonlinear elimination
sinfu.mm.out<-function(PKtime,Vm,Km,Vd,defun,par1,par2,par3,Dose,i)
{
  time<-PKtime$time
  parms<-c(Vm=Vm,Km=Km,Vd=Vd)  
  C1.lsoda<-data.frame(lsoda(0,c(0,time),defun,parms))    
  cat("\n")
  sim1<-Vm
  sim2<-Km
  sim3<-Vd
  sim<-matrix(c(sim1,sim2,sim3,par1,par2,par3),3,2)
  dimnames(sim)<-list(c("Vm","Km","Vd"),c("Value","Original"))
  show(sim)    
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),2])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for extravascular first order absorption with/without lag time
sfirst1.out<-function(PKtime,ka,kel,Vd,defun,par1,par2,par3,Dose,i)  
{   
  time<-PKtime$time
  parms<-c(ka=ka,kel=kel,Vd=Vd)  
  C1.lsoda<-data.frame(lsoda(c(Dose,0),c(0,time), defun, parms))  
  cat("\n\n")
  sim1<-ka
  sim2<-kel
  sim3<-Vd
  sim<-matrix(c(sim1,sim2,sim3,par1,par2,par3),3,2)
  dimnames(sim)<-list(c("ka","kel","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),3])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}  

#for extravascular first order absorption with/without lag time nonlinear elimination
sfirst.mm.out<-function(PKtime,ka,Vm,Km,Vd,defun,par1,par2,par3,par4,Dose,i)  
{       
  time<-PKtime$time
  parms<-c(ka=ka,Vm=Vm,Km=Km,Vd=Vd) 
  C1.lsoda<-data.frame(lsoda(c(Dose,0),c(0,time), defun, parms)) 
  cat("\n\n")
  sim1<-ka
  sim2<-Vm
  sim3<-Km
  sim4<-Vd
  sim<-matrix(c(sim1,sim2,sim3,sim4,par1,par2,par3,par4),4,2)
  dimnames(sim)<-list(c("ka","Vm","Km","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),3])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for extravascular zero order absorption without lag time
szero.out<-function(PKtime,Tabs,kel,Vd,defun,par1,par2,par3,Dose,i)    
{
  time<-PKtime$time
  parms<-c(Tabs=Tabs,kel=kel,Vd=Vd)  
  C1.lsoda<-data.frame(lsoda(0, c(0,time), defun, parms))   
  cat("\n")
  sim1<-Tabs
  sim2<-kel
  sim3<-Vd
  sim<-matrix(c(sim1,sim2,sim3,par1,par2,par3),3,2)
  dimnames(sim)<-list(c("Tabs","kel","Vd"),c("Value","Original"))
  show(sim)  
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),2])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for extravascular zero order absorption without lag time nonlinear elimination
szero.mm.out<-function(PKtime,Tabs,Vm,Km,Vd,defun,par1,par2,par3,par4,Dose,i)
{                      
  time<-PKtime$time
  parms<-c(Tabs=Tabs,Vm=Vm,Km=Km,Vd=Vd) 
  C1.lsoda<-data.frame(lsoda(0,c(0,time), defun, parms))   
  cat("\n\n")
  sim1<-Tabs
  sim2<-Vm
  sim3<-Km
  sim4<-Vd
  sim<-matrix(c(sim1,sim2,sim3,sim4,par1,par2,par3,par4),4,2)
  dimnames(sim)<-list(c("Tabs","Vm","Km","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),2])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for one exponential term
smacro.one.out<-function(PKtime,A,a,defun,par1,par2,Dose,i)
{         
  time<-PKtime$time
  defun<- A*exp(-a*time)
  output<-data.frame(time,defun) 
  cat("\n")
  sim1<-A
  sim2<-a
  sim<-matrix(c(sim1,sim2,par1,par2),2,2)
  dimnames(sim)<-list(c("A","a"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  PKindex<-data.frame(i,output)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-PKindex[,2]
  y<-PKindex[,3]
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for two exponential term
smacro.two.out<-function(PKtime,A,a,B,b,defun,par1,par2,par3,par4,Dose,i)      
{   
  time<-PKtime$time
  defun<- A*exp(-a*time)+B*exp(-b*time)
  output<-data.frame(time,defun) 
  cat("\n")
  sim1<-A
  sim2<-a
  sim3<-B
  sim4<-b
  sim<-matrix(c(sim1,sim2,sim3,sim4,par1,par2,par3,par4),4,2)
  dimnames(sim)<-list(c("A","a","B","b"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  PKindex<-data.frame(i,output)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-PKindex[,2]
  y<-PKindex[,3]
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for three exponential term
smacro.three.out<-function(PKtime,A,a,B,b,C,c,defun,par1,par2,par3,par4,par5,par6,Dose,i)      
{         
  time<-PKtime$time
  defun<- A*exp(-a*time)+B*exp(-b*time)+C*exp(-c*time)
  output<-data.frame(time,defun) 
  cat("\n")
  sim1<-A
  sim2<-a
  sim3<-B
  sim4<-b
  sim5<-C
  sim6<-c
  sim<-matrix(c(sim1,sim2,sim3,sim4,sim5,sim6,par1,par2,par3,par4,par5,par6),6,2)
  dimnames(sim)<-list(c("A","a","B","b","C","c"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  PKindex<-data.frame(i,output)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-PKindex[,2]
  y<-PKindex[,3]
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for two compartment iv bolus
sbolus2.out<-function(PKtime,kel,k12,k21,Vd,defun,par1,par2,par3,par4,Dose,i)
{ 
  time<-PKtime$time
  parms<-c(kel=kel,k12=k12,k21=k21,Vd=Vd) 
  C1.lsoda<-data.frame(lsoda(c(Dose/Vd,0),c(0,time),defun,parms)) 
  cat("\n")
  sim1<-kel
  sim2<-k12
  sim3<-k21
  sim4<-Vd
  sim<-matrix(c(sim1,sim2,sim3,sim4,par1,par2,par3,par4),4,2)
  dimnames(sim)<-list(c("kel","k12","k21","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),2])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for two compartment iv infusion
sinfu2.out<-function(PKtime,kel,k12,k21,Vd,defun,par1,par2,par3,par4,Dose,i)
{       
  time<-PKtime$time
  parms<-c(kel=kel,k12=k12,k21=k21,Vd=Vd) 
  C1.lsoda<-data.frame(lsoda(c(0,0),c(0,time),defun,parms)) 
  cat("\n")
  sim1<-kel
  sim2<-k12
  sim3<-k21
  sim4<-Vd
  sim<-matrix(c(sim1,sim2,sim3,sim4,par1,par2,par3,par4),4,2)
  dimnames(sim)<-list(c("kel","k12","k21","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),2])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}

#for two compartment extravascular first order absorption
sfirst2.out<-function(PKtime,ka,kel,k12,k21,Vd,defun,par1,par2,par3,par4,par5,Dose,i)
{                   
  time<-PKtime$time
  parms<-c(ka=ka,kel=kel,k12=k12,k21=k21,Vd=Vd) 
  C1.lsoda<-data.frame(lsoda(c(Dose,0,0),c(0,time),defun,parms)) 
  cat("\n")
  sim1<-ka
  sim2<-kel
  sim3<-k12
  sim4<-k21
  sim5<-Vd
  sim<-matrix(c(sim1,sim2,sim3,sim4,sim5,par1,par2,par3,par4,par5),5,2)
  dimnames(sim)<-list(c("ka","kel","k12","k21","Vd"),c("Value","Original"))
  show(sim)
  cat("\n\n<< Output >>\n\n")
  good<-ifelse(C1.lsoda[2:(length(time)+1),2]<=0,0,C1.lsoda[2:(length(time)+1),3])
  PKindex<-data.frame(i,C1.lsoda[2:(length(time)+1),1],good)
  colnames(PKindex)<-list("Subject","time","conc")
  show(PKindex)
  x<-C1.lsoda[2:(length(time)+1),1]
  y<-good
  plotting.sim(i,x,y)
  savefile(PKindex) 
}
